--- codec_g726.c.1_6_2_1	2009-11-10 20:09:47.000000000 +0000
+++ codec_g726.c	2010-01-27 14:10:43.000000000 +0000
@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 229285 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/lock.h"
 #include "asterisk/linkedlists.h"
@@ -769,13 +769,29 @@
 	return 0;
 }
 
+/*! \brief convert G726-32 RFC3551 packed data into AAL2 packed data (or vice-versa) */
+static int g726tog726aal2_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
+{
+	unsigned char *src = f->data.ptr;
+	unsigned char *dst = pvt->outbuf.uc + pvt->samples;
+	unsigned int i;
+
+	for (i = 0; i < f->datalen; i++)
+		*dst++ = ((src[i] & 0xf) << 4) | (src[i] >> 4);
+
+	pvt->samples += f->samples;
+	pvt->datalen += f->samples; /* 1 byte/sample */
+
+	return 0;
+}
+
 static struct ast_translator g726tolin = {
 	.name = "g726tolin",
 	.srcfmt = AST_FORMAT_G726,
 	.dstfmt = AST_FORMAT_SLINEAR,
 	.newpvt = lintog726_new,	/* same for both directions */
 	.framein = g726tolin_framein,
-	.sample = g726_sample,
+	.sample = g726tolin_sample,
 	.desc_size = sizeof(struct g726_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES * 2,
@@ -788,7 +804,7 @@
 	.dstfmt = AST_FORMAT_G726,
 	.newpvt = lintog726_new,	/* same for both directions */
 	.framein = lintog726_framein,
-	.sample = slin8_sample,
+	.sample = lintog726_sample,
 	.desc_size = sizeof(struct g726_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES/2,
@@ -800,7 +816,7 @@
 	.dstfmt = AST_FORMAT_SLINEAR,
 	.newpvt = lintog726_new,	/* same for both directions */
 	.framein = g726aal2tolin_framein,
-	.sample = g726_sample,
+	.sample = g726tolin_sample,
 	.desc_size = sizeof(struct g726_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES * 2,
@@ -813,12 +829,32 @@
 	.dstfmt = AST_FORMAT_G726_AAL2,
 	.newpvt = lintog726_new,	/* same for both directions */
 	.framein = lintog726aal2_framein,
-	.sample = slin8_sample,
+	.sample = lintog726_sample,
 	.desc_size = sizeof(struct g726_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES / 2,
 };
 
+static struct ast_translator g726tog726aal2 = {
+	.name = "g726tog726aal2",
+	.srcfmt = AST_FORMAT_G726,
+	.dstfmt = AST_FORMAT_G726_AAL2,
+	.framein = g726tog726aal2_framein,	/* same for both directions */
+	.sample = lintog726_sample,
+	.buffer_samples = BUFFER_SAMPLES,
+	.buf_size = BUFFER_SAMPLES,
+};
+
+static struct ast_translator g726aal2tog726 = {
+	.name = "g726aal2tog726",
+	.srcfmt = AST_FORMAT_G726_AAL2,
+	.dstfmt = AST_FORMAT_G726,
+	.framein = g726tog726aal2_framein,	/* same for both directions */
+	.sample = lintog726_sample,
+	.buffer_samples = BUFFER_SAMPLES,
+	.buf_size = BUFFER_SAMPLES,
+};
+
 static int parse_config(int reload)
 {
 	struct ast_variable *var;
@@ -855,6 +891,9 @@
 	res |= ast_unregister_translator(&g726aal2tolin);
 	res |= ast_unregister_translator(&lintog726aal2);
 
+	res |= ast_unregister_translator(&g726aal2tog726);
+	res |= ast_unregister_translator(&g726tog726aal2);
+
 	return res;
 }
 
@@ -872,6 +911,9 @@
 	res |= ast_register_translator(&g726aal2tolin);
 	res |= ast_register_translator(&lintog726aal2);
 
+	res |= ast_register_translator(&g726aal2tog726);
+	res |= ast_register_translator(&g726tog726aal2);
+
 	if (res) {
 		unload_module();
 		return AST_MODULE_LOAD_FAILURE;
